////////////////////////////////////////////////////////////////////////////// 
//
//  ChartContainer.h
//
///////////////////////////////////////////////////////////////////////////////

#include "ChartDef.h"

#pragma once

///////////////////////////////////////////////////////////////////////////////
// Struct for notifications from CChartDataView

typedef struct  
  {
    NMHDR nmhdr;
    bool bAdd;
    int chartIdx;
    size_t dataID;
    PointT<double> dataPntD;
  } STRUCT_NOTIFY, *LP_STRUCT_NOTIFY;

///////////////////////////////////////////////////////////////////////////////
// class CChartContainer

class CChart;
class CDataWnd;
class CChartDataView;

class MatrixD;

class CChartContainer : public CWnd
{
	DECLARE_DYNAMIC(CChartContainer)

  friend class CDataView;

public:
	CChartContainer(string_t name = string_t(_T("")));
	virtual ~CChartContainer();

private:
  CChartContainer(const CChartContainer& src);
public:
  CChartContainer& operator =(const CChartContainer& rhs);
  bool CreateChartCtrlWnd(DWORD dwExStyle, DWORD dwStyle, const CRect& wndRect, 
                                                                    CWnd* pParent, UINT nID);

public: 
  CChartContainer* CloneChartContainer(string_t name = string_t(_T("")), bool bCloneState = false);
protected:
  CChart* CloneChart(int chartIdx, bool bCloneState = false);

// Access to colors, get
public:
  Gdiplus::Color GetBkColor(void) const {return m_colBkground;}
  Gdiplus::Color GetBorderColor(void) const {return m_colBorder;}
  Gdiplus::Color GetAxesColor(void) const { return m_colAxis;}
  Gdiplus::Color GetZoomBorderColor(void) const { return m_colZoomBorder;}
  Gdiplus::Color GetLegPntsColor(void) const {return m_colLegPnts;}
  Gdiplus::Color GetDataViewPntsColor(void) const { return m_colDataViewPnts;}
  Gdiplus::Color GetLegBkgndColor(void) const {return m_colLegBkgnd;}
  Gdiplus::Color GetLegBorderColor(void) const {return m_colLegBorder;}
  Gdiplus::Color GetGridColor(void) const {return m_colGrid;}
// Access to colors, set
  void SetBkColor(Gdiplus::Color col, bool bRedraw = false);
  void SetBorderColor(Gdiplus::Color col, bool bRedraw = false);
  void SetAxesColor(Gdiplus::Color col, bool bRedraw = false);
  void SetZoomBorderColor(Gdiplus::Color col);
  void SetLegPntsColor(Gdiplus::Color col, bool bRedraw = false);
  void SetDataViewPntsColor(Gdiplus::Color col, bool bRedraw = false);
  void SetLegBkgndColor(Gdiplus::Color col, bool bRedraw = false);
  void SetLegBorderColor(Gdiplus::Color col, bool bRedraw = false);
  void SetGridColor(Gdiplus::Color col, bool bRedraw = false);

// Access functions: access to m_startX, m_endX, m_minExtY, m_maxExtY
  double GetStartX(void) const {return m_startX;}
  double GetEndX(void) const {return m_endX;}
  double GetInitialStartX(void) const;  // Returns front of m_dqHist 
  double GetInitialEndX(void) const;    // or m_startX if there is no history

  double GetMinY(void) const {return m_minExtY;}
  double GetMaxY(void) const {return m_maxExtY;}

  double GetScX(void) const {return m_scX;}
  double GetScY(void) const {return m_scY;}

// To set members to min/max of arguments and m_startX, m_endX, m_minExtY, m_maxExtY
  void UpdateExtX(double minExtX, double maxExtX, bool bRedraw = false);
  void UpdateExtY(double minExtY, double maxExtY, bool bRedraw = false);

// Access to precision (number of significant digits)
  int GetContainerPrecisionX(void) const {return m_precision;}
  void SetContainerPrecision(int precision, bool bRedraw = false);

  int GetDataViewChartIdx(void) const { return m_dataViewChartIdx;}
  void SetDataViewChartIdx(int chartIdx) {m_dataViewChartIdx = chartIdx;}
  const MAP_SELPNTSD* GetDataViewSelPntsD(void) const {return &m_mapDataViewPntsD;}
  size_t SetDataViewSelPnts(int chartIdx, const MAP_SELPNTSD& mapSelPntsD,
                                                        V_CHARTDATAD* pChartData = NULL);
  bool ClearDataViewChartIdx(void);

  string_t GetContainerName(void) const {return m_name;}
  void SetContainerName(string_t name) { m_name = name.empty() ? string_t(_T("X")) : name;}

  string_t GetAxisXName(void) const {return m_labelX;}
  void SetAxisXName(string_t nameX, bool bRedraw = false); 
// X-axis string formatting function (container)
  val_label_str_fn GetLabXValStrFnPtr(void) const {return m_pLabValStrFnPtr;}
  void SetLabXValStrFn(val_label_str_fn pLabValStrFn, bool bRedraw = false);

  bool IsContainerEmpty(void) const {return m_mapCharts.empty();}
  size_t GetContainerSize(void) const {return m_mapCharts.size();}
  MAP_CHARTS::iterator GetFirstChart(void) {return m_mapCharts.begin();}
  MAP_CHARTS::const_iterator GetFirstChart(void) const {return m_mapCharts.cbegin();}
  MAP_CHARTS::iterator GetChartsEnd(void) {return m_mapCharts.end();}
  MAP_CHARTS::const_iterator GetChartsEnd(void) const {return m_mapCharts.cend();}
  MAP_CHARTS::iterator FindChart(int chartIdx) {return m_mapCharts.find(chartIdx);}
  MAP_CHARTS::const_iterator FindChart(int chartIdx) const {return m_mapCharts.find(chartIdx);}

  size_t CopyHistory(V_HIST& vHist) const
                { vHist.assign(m_vHist.cbegin(), m_vHist.cend()); return vHist.size();}
  size_t ResetHistory(const V_HIST& vHist) 
                {m_vHist.assign(vHist.cbegin(), vHist.cend()); return m_vHist.size();}


protected:
  CH_MODE GetChMode(void) const {return m_chMode;}

// Access to the member m_pMatrix:
  MatrixD* GetTransformMatrixD(void) const {return m_pMatrixD;}
// Calculates the transformation matrix for given client rectangle 
  MatrixD* GetTransformMatrixD(double startX, double endX, double minY, double maxY);
  void SetTransformMatrixD(MatrixD* pMatrixD); 
  void SetTransformMatrixD(void); 
 
// Iterates over charts in the container and returns min/max X or Y. Does not change the container's
// m_startX, m_endX, m_minExtY, m_maxExtY
public:
  PAIR_DBLS GetMinMaxExtX(void) const;
  PAIR_DBLS GetMinMaxExtY(void) const;

// To restore intial state of the container
  void UndoHistStep(bool bRedraw = false);
// Called by parent or owner when parent's size is changing
  void OnChangedSize(int cx, int cy);

// Drawing the container
  void RefreshWnd(void);  // Redraws the container, excludes dataWnd

// Save/Load to/from XML files
  HRESULT SaveChartData(void);
  static HRESULT SaveChartData(const string_t pathName, const CChartContainer* pContainer, 
                                                                      const string_t chartName);
  HRESULT LoadCharts(LPCTSTR fileName, const MAP_CHARTCOLS& mapContent);
  HRESULT GetChartNamesFromXMLFile(LPCTSTR fileName, MAP_CHARTCOLS& mapContent);
  HRESULT GetChartNamesFromXMLFile(LPCTSTR fileName, MAP_NAMES& mapNames);
// TODO: constantnes of the get functions
  bool ExportChartData(string_t chartName, V_CHARTDATAD& vDataPnts) const;
  bool ExportChartData(string_t chartName, std::vector<std::pair<double, double> >& vPairs) const;
  bool ExportChartData(string_t chartName, std::vector<double>& vX, std::vector<double>& vY) const;

///////////////////////////////////////////////////////////////////////////////
// Access to chart and container state variables

// Returns the chart's minmax for X if bY == false, or for Y (if true), and true.
// in the third element. Returns false in the third element if the chart does not exist. 
  std::tuple<double, double, bool> GetChartBoundaries(int chartIdx, bool bY = false) const; 
// Visibility
protected:
  CChart* GetVisibleChart(int chartIdx) const;  // Returns pointer to the (first) visible chart or NULL
public:
  bool IsChartVisible(int chartIdx) const;
  bool SetChartVisibility(int chartIdx, bool bVisible, bool bRedraw = false); 
  int ToggleChartVisibility(int chartIdx);// Returns 1 if set visible, 0 if not,
                                          // -1 if the chart does not exist
// Chart Pen Width  
  std::pair<float, bool> GetChartPenWidth(int chartIdx) const;
  bool UpdateChartPenWidth(int chartIdx, float penWidth, bool bRedraw = false);// Changes all or selected pen width

// Tension
  std::pair<float, bool> GetChartTension(int chartIdx) const;
  bool SetChartTension(int chartIdx, float tension, bool bRedraw = false);

// Show points
  std::pair<bool, bool> AreChartPntsAllowed(int chartIdx) const;
  bool AllowChartPnts(int chartIdx, bool bAllowed, bool bRedraw = false);

// Chart Color:Returns false as the second member of the pair if the chart does not exist
  std::pair<Gdiplus::Color, bool> GetChartColor(int chartIdx) const;
  bool SetChartColor(int chartIdx, Gdiplus::Color col, bool bRedraw = false);

// Chart Dash Style
  std::pair<Gdiplus::DashStyle, bool> GetChartDashStyle(int chartIdx) const;
  bool SetChartDashStyle(int chartIdx, Gdiplus::DashStyle dashStyle, bool bRedraw = false);

// Get the selected chart
  int HasSelectedChart(void) const;       // Returns chartIdx or -1
  int SelectChart(int chartIdx, bool bRedraw = false);
  int SelectFirstVisibleChart(bool bRedraw = false);
  int SelectChartByMouseClick(CPoint pnt, bool bRedtaw = false);

// Chart Y axis precision
  int GetChartPrecisionY(int chartIdx) const;
  bool SetChartPrecisionY(int chartIdx, int precisionY, bool bRedraw = false);

// Chart Name
  string_t GetChartName(int chartIdx) const;
  bool ChangeChartName(int chartIdx, string_t name, bool bRedraw = false);

// Chart Y data name
  string_t GetChartYValName(int chartIdx) const;
  bool ChangeChartYValName(int chartIdx, string_t nameY, bool bRedraw = false);

// Function to get YVal string
  val_label_str_fn GetLabYValStrFnPtr(int chartIdx) const;
  bool SetLabYValStrFn(int chartIdx, val_label_str_fn m_pLabYValStrFn, bool bRedraw = false);

// Get Chart: Returns chart pointer or NULL
protected:
  CChart* GetChart(int chartIdx) const;        
  CChart* FindChartByName(string_t nameStr) const;
public:   
  V_CHARTDATAD* GetChartData(int chartDIdx);
  int GetChartIdx(string_t chartName) const;

// Local scale Y (m_locScY)
  std::pair<double, bool> GetChartLocScaleY(int chartIdx) const;
  bool UpdateChartLocScaleY(int chartIdx, double locScY, bool bRedraw = false);

  bool ZoomMoveContainer(double startX, double endX);

protected:
  CH_MODE ZoomContainer(double startX, double endX, bool bRedraw = false);
// If deltaX < 0 the chart curves are moving to the left
  CH_MODE MoveContainerCharts(double deltaX, bool bRedraw = false);

// Notification on actions from the popup menu
  LRESULT SendNotification(UINT code, int chartIdx = -1);

////////////////////////////////////////////////////////////////////////////////////
// Chart interface functions

public:
 int AddChart(bool bVisible, bool bShowPnts, string_t label, string_t labelY, int precisionY, 
              Gdiplus::DashStyle dashStyle, float penWidth, float tension, 
              Gdiplus::Color colChart, V_CHARTDATAD& vData, bool bRedraw = false);
 // Overload for time series
 int AddChart(bool bVisible, bool bShowPnts, string_t label, string_t labelY, int precisionY, 
                Gdiplus::DashStyle dashStyle, float penWidth, float tension, 
                Gdiplus::Color colChart, std::vector<double>& vTmSeries, bool bRedraw = false);
 // Overload for vector of pairs
 int AddChart(bool bVisible, bool bShowPnts, string_t label, string_t labelY, int precisionY,
                Gdiplus::DashStyle dashStyle, float penWidth, float tension, 
                Gdiplus::Color colChart, std::vector<std::pair<double, double> >& vXYData, 
                bool bRedraw = false);
// Overload for vectors of doubles
 int AddChart(bool bVisible, bool bShowPnts, string_t label, string_t labelY, int precisionY, 
                Gdiplus::DashStyle dashStyle, float penWidth, float tension, 
                Gdiplus::Color colChart, std::vector<double>& vX, std::vector<double>& vY, 
                bool bRedraw = false);
 
 // Appends points to the end of chart data vector. Discards all new points with X < m_endX
  bool AppendChartData(int chartIdx, V_CHARTDATAD& vData, bool bUpdate = false);
// Overload for time series
  bool AppendChartData(int chartIdx,  std::vector<double>& vTmSeries, bool bUpdate = false);
// Overload for vector of pairs
  bool AppendChartData(int chartIdx,  std::vector<std::pair<double, double> >& vXYData,
                                                                      bool bUpdate = false);
// Overload for vector of doubles
  bool AppendChartData(int chartIdx,  std::vector<double>& vX, std::vector<double>& vY,
                                                                      bool bUpdate = false);
// Replaces chart data vector
  bool ReplaceChartData(int chartIdx, V_CHARTDATAD& vData, bool bClip, 
                                           bool bVerbose = false, bool bRedraw = false);
// Overload for time series
  bool ReplaceChartData(int chartIdx,  std::vector<double>& vTmSeries,
                                           bool bVerbose = false, bool bRedraw = false);
// Overload for vector of pairs
  bool ReplaceChartData(int chartIdx,  std::vector<std::pair<double, double> >& vXYData,
                                              bool bVerbose = false, bool bRedraw = false);
// Overload for vector of doubles
  bool ReplaceChartData(int chartIdx,  std::vector<double>& vX, std::vector<double>& vY,
                                              bool bVerbose = false, bool bRedraw = false);

  
  void ResetCharts(void);
  void ResetChartContainer(bool bResetCols = false);
  bool TruncateChart(int chartIdx, double startX, double endX, bool bUpdateExt = false, bool bRedraw = false);
  bool RemoveChart(int chartIdx, bool bCorrectMinMax, bool bRedraw);

  void UpdateDataViewPnts(int chartIdx, size_t dataID, PointD dataPntD, bool bAdd);
  void UpdateDataViewPnts(int chartIdx, const MAP_SELPNTSD& mapSelPntsD);

///////////////////////////////////////////////////////////////////////////////
//  Legend functions
public:
  PointD GetDataLegPntD(void) const { return m_dataLegPntD;}
// To set the SelData pnt from a program
  CDataWnd* SetDataLegPntD(PointD pntD, bool bRedraw = false);
  bool IsLabWndVisible(bool bLeg) const;  // true - m_pLegWnd, false - m_pDataWnd
  bool IsLabWndExist(bool bLeg) const;    // same
protected:
  CDataWnd* ShowDataLegend(CPoint pnt);
//TODO: Make these two function public; provide the public function for axis label and data view
//TODO: Make switch to disable the context menu
  CDataWnd* ShowDataLegend(void); // Uses m_dataLegPntD
  CDataWnd* ShowNamesLegend(void);
// Uses the selected points to correct the data label and shows it in the appropriate position
  void UpdateDataLegend(bool bChangeMatrix); 
  void CopyDataLegendWnd(void);
  CDataWnd* DestroyLabWnd(CDataWnd* pDataWnd);
  void HideLabWnds(bool bRedraw = false);

// Clear chart data view
public:
  void DestroyChartDataView(void);

public:
// Updates container wnd and child windows
  void UpdateContainerWnds(int chartIdx = -1, bool bMatrix = false, DATAVIEW_FLAGS dataChange = F_NODATACHANGE);   
// To get max chart Idx in the mapCharts
  int GetMaxChartIdx(void) const;
protected:
// To define invalidated region
  CRect GetExtXClientRect(double startX, double endX);
// Translate the mouse coordinates to the data space (CPoint to PointD)
  void MousePntToPntD(CPoint pnt, PointD& dataPntD, MatrixD* pMatrixD);

  double UpdateScaleX(const float drawRectWidth, double startX, double endX); 
  double UpdateScaleY(const float drawRectHeight, double minY, double maxY);
// Calculates scX, scY but does not set m_scX, m_scY
  std::pair<double, double> UpdateScales(const Gdiplus::RectF drawRectF, 
                                        double minX, double maxX, double minY, double maxY);

// Zoom on mouse left clicks + MK_SHIFT
  double BeginZooming(CPoint pnt, bool bDraw);                // First click
  double PrepareZoom(CPoint pnt, double startX, bool bDraw);  // Second click
// Restores the state to ZOOM or FULLX after interruption between the first and second zoom clicks
  CH_MODE RestoreMode(void);

// Legend 
  size_t PrepareDataLegend(CPoint pnt, MAP_LABSTR& mapLabels, MAP_SELPNTSD& mapSelPntsD);
  size_t PrepareDataLegend(PointD origPntD, double epsX, MAP_LABSTR& mapLabels, 
                                                    MAP_SELPNTSD& mapSelPntsD, bool bChangeMatrix);
  size_t PrepareNamesLegend(MAP_LABSTR& mapLabels);
  size_t UpdateDataLegend(MAP_SELPNTSD& mapSelPntsD, MAP_LABSTR& m_mapLabs); 

// To use with UpdateDataLegend
// Gets axis X pos AX_YPOS (first) and axis X float offsY(second)
  AX_XPOS GetXAxisPos(void) const {return m_axXPos;} 
  PAIR_XAXPOS GetXAxisPos(Gdiplus::RectF rChartF, double minY, double maxY);
// Gets axis Y pos AX_XPOS (first) and axis Y float offsX(second)
  AX_YPOS GetYAxisPos(void) const {return m_axYPos;}
  PAIR_YAXPOS GetYAxisPos(Gdiplus::RectF rChartF, double startX, double endX);

  float GetAxisOffsetX(void) const {return m_axOffsXF;}

// To show data view
  bool ShowDataView(CChart* chartPtr, bool bClearMap, bool bRefresh = true);
  bool UpdateDataView(int chartIdx, DATAVIEW_FLAGS flagsData);
// Overload that checks whether the chart has data view, and exists
//  bool UpdateDataView(int chartIdx, bool bClearMap, bool bRefresh);
// To use when selected chart is appended, truncated or its data were replaced
  size_t UpdateDataViewPntsDMap(void);

// This returns chartPtr->m_loScY as a first, and deltaLocScY as a second; delta = 1% of Y range
 PAIR_DBLS GetLocScYDelta(CChart* chartPtr);

  void MoveChartsByArrowKeys(UINT nChar);
  void ChangeLocScaleYByArrowKeys(UINT nChar);

// Drawing functions
  Gdiplus::PointF DrawAxes(Gdiplus::RectF rClientF, double startX, double endX, double minY, 
         double maxY, Gdiplus::Color colAxis, Gdiplus::Graphics* grPtr, float dpiRatio = 1.0f);
  void DrawBkground(Gdiplus::RectF rClientF, Gdiplus::Color colBkground, 
                              Gdiplus::Color colBorder, Gdiplus::Graphics* grPtr);
  bool DrawSelPntsD(CChart* chartPtr, const MAP_SELPNTSD& mapSelPntsD, 
                Gdiplus::GraphicsPath* grPathPtr, Gdiplus::Graphics* grPtr, float dpiRatio = 1.0f);
  bool DrawDataViewPntsD(const MAP_SELPNTSD& mapDataViewPntsD, double startX, double endX, 
              Gdiplus::GraphicsPath* grPathPtr, Gdiplus::Graphics* grPtr, float dpiRatio = 1.0f);
  void DrawSelLine(Gdiplus::RectF clRF, Gdiplus::Graphics* grPtr, float dpiRatio = 1.0f);
  void DrawAxXBoundaryStr(Gdiplus::RectF boundRF, Gdiplus::Graphics* grPtr, float dpiRtio = 1.0f);
  void DrawZoomBorder(CPoint pnt);
  void DrawGrid(Gdiplus::RectF boundRF, Gdiplus::PointF coordOrigF, 
                                                 Gdiplus::Graphics* grPtr, float dpiRatio = 1.0f);
// Print's helpers
  int GetScreenDpi(void);

  TUPLE_PRNLEGLAYOUT CalcNamesPntLayout(Gdiplus::RectF namesRF, const MAP_PRNDATA& mapPrnData, 
                                                          Gdiplus::Graphics* grPtr, float dpiRatio);
  void PrintCrossLine(Gdiplus::PointF pntStartF, Gdiplus::PointF pntEndF, Gdiplus::SizeF sizeCrossF, 
                                        Gdiplus::Pen* penPtr, Gdiplus::Graphics* grPtr, float dpiRatio); 
  size_t PrintChartNames(Gdiplus::RectF namesRF, const MAP_PRNDATA& mapPrnData, 
                         const TUPLE_PRNLEGLAYOUT& tuple_prnLayout, 
                         Gdiplus::Graphics* grPtr, float dpiRatio, size_t mapOffs = 0);

  string_t PrintPageHeader(Gdiplus::RectF rGdiF, const CChartContainer* pContainer, string_t tmStr, 
                                                                            Gdiplus::Graphics* grPtr);
  static
    void PrintCharts(CChartContainer* pContainer, int scrDpiX, HDC printDC);

protected:
	DECLARE_MESSAGE_MAP()

// Message handlers
  afx_msg void OnPaint(void);

  afx_msg UINT OnGetDlgCode();

  afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
  afx_msg void OnLButtonUp(UINT nFlags, CPoint point);

  afx_msg void OnMButtonDown(UINT nFlags, CPoint point);
  afx_msg BOOL OnMouseWheel(UINT nFlags, short zDelta, CPoint pt);

  afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);

  afx_msg BOOL OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message);

  afx_msg void OnTimer(UINT_PTR nIDEvent);

  afx_msg void OnKillFocus(CWnd* pNewWnd);
  afx_msg void OnSetFocus(CWnd* pOldWnd);

// Context Menu Command Handlers
  afx_msg void OnContextMenu(CWnd* /*pWnd*/, CPoint /*point*/);
  afx_msg void OnMnShowLegend(void);
  afx_msg void OnMnShowAxBoundaries(void);
  afx_msg void OnMnRefresh(void);
  afx_msg void OnMnViewData(void);
  afx_msg void OnMnSaveCharts(void);
  afx_msg void OnMnPrintCharts(void);
  afx_msg void OnMnZoomIn(void);  // ZoomIn from the context menu
  afx_msg void OnMnMoveRight(void);
  afx_msg void OnMnMoveLeft(void);
  afx_msg void OnMnUndoHistStep(void);
  afx_msg void OnMnEnableTracking(void);
  afx_msg void OnMnEnableLocScale(void);
  afx_msg void OnMnShowPnts(void);
  afx_msg void OnMnShowChart(UINT nCmdID);

#ifdef _TEST_
  afx_msg void OnMnTest(void);
#endif

// Data Members
protected:
  Gdiplus::Color m_colBkground;
  Gdiplus::Color m_colBorder;
  Gdiplus::Color m_colAxis;
  Gdiplus::Color m_colZoomBorder;
  Gdiplus::Color m_colLegPnts;
  Gdiplus::Color m_colDataViewPnts;
  Gdiplus::Color m_colLegBkgnd;
  Gdiplus::Color m_colLegBorder;
  Gdiplus::Color m_colGrid;

  double m_minExtY;       // Min coordinate of the Y axis
  double m_maxExtY;       // Max coordinate of the Y axis
  double m_startX;        // Start X coord
  double m_endX;          // Last X coord (not included)
  float  m_axOffsXF;      // Y Offset of X axis
  AX_XPOS m_axXPos;       // Position of zero on the X axis 
  AX_YPOS m_axYPos;       // Position of zero on the Y axis
  double m_scX;
  double m_scY;
  bool m_bTracking;       // Cross cursor and data legend are enabled
  bool m_bSelEnabled;     // Local Scaling is enabled
  bool m_bAxBoundaries;   // Show the X axis left and right boundaries
  int  m_chartSelIdx;      // ID of the chart selected to scale Y locally
  int  m_dataViewChartIdx; // ID of the chart displayed in data view
  int  m_precision;       // Number of decimal digits after decimal point
                          // for data labels
  double m_tmpZX;         // Temporary placeholder for zoom startX 

  MatrixD* m_pMatrixD;    // Transform matrix from the data pnt space to the client coord

  PointD m_dataLegPntD;   // Transformed to the data point space point of sel line
  double m_epsX;          // Neighbourhood to search for nearest point

protected:
  V_HIST m_vHist;         // Zoom & pane history
  MAP_CHARTS m_mapCharts; // Container's content
  MAP_SELPNTSD m_mapSelPntsD;       // Points nearest to the sel line for data window
  MAP_SELPNTSD m_mapDataViewPntsD;  // Chart's data points selected in the data view window 

  CDataWnd* m_pDataWnd;
  CDataWnd* m_pLegWnd;
  CChartDataView* m_pChartDataView;

protected:
  CH_MODE m_chMode;     
  string_t m_name;        // Container's name
  string_t m_labelX;      // X-axis name

  val_label_str_fn m_pLabValStrFnPtr;
};






